% *************************************************************************
% Problem:  Acrobot ROA - dual SOS problem
% Requires: Gloptipoly 3, Yalmip + SDP solver (tested with MOSEK and SeDuMi)
% Authors:  Milan Korda, Didier Henrion, 2013
% *************************************************************************
% Warning: takes a very long time with SeDuMi

close all
clear all
addpath(genpath('./functions'))

% SDP solver
SDPsolver = 'mosek'; % or sedumi

d = 6; % degree of the polynomial
n = 4; % dimension of the state
m = 2; % dimension of the control

% Variables
t = sdpvar(1,1);
x = sdpvar(n,1);
u = sdpvar(m,1);

% Polynomials v(t,x) and w(x)
[v, cv] = polynomial([t;x],d);
[w, cw] = polynomial(x,d);

% Final time
T = 2;

% Load dynamics (after Taylor expansion and scaling of the constraints to
% unit boxes) + the scalings
load('AcrobotDynamics.mat','f_str','scale_x','scale_u'); 
f = [];
for i = 1:numel(f_str)
    f = [f ; eval(f_str{i})];
end
f = f * T;

% State constraint X = [-1,1]^4 (scaled)
xb = [1 1 1 1];
gx1 = xb(1)^2 - x(1)^2;
gx2 = xb(2)^2 - x(2)^2;
gx3 = xb(3)^2 - x(3)^2;
gx4 = xb(4)^2 - x(4)^2;


% Input constraints U = [-1,1]^2 (scaled)
% If ub = [0 1] => just the middle joint, if  ub = [1 1] => both joints
ub = [0 1];
gu1 = ub(1)^2 - u(1)^2;
gu2 = ub(2)^2 - u(2)^2;

% Target set XT = {x : gxT(x) = 0}
gxT = x'*x;

% Sum of squares multipliers
[s1 c1] = polynomial([t;x;u],d-2);
[s2 c2] = polynomial([t;x;u],d-2);
[s3 c3] = polynomial([t;x;u],d-2);
[s4 c4] = polynomial([t;x;u],d-2);
[s5 c5] = polynomial([t;x;u],d-2);
[s14 c14] = polynomial([t;x;u],d-2);
[st ct] = polynomial([t;x;u],d-2);
[s6 c6] = polynomial(x,d-2);
[s7 c7] = polynomial(x,d-2);
[s8 c8] = polynomial(x,d-2);
[s9 c9] = polynomial(x,d-2);
[s10 c10] = polynomial(x,d-2);
[s11 c11] = polynomial(x,d-2);
[s12 c12] = polynomial(x,d-2);
[s13 c13] = polynomial(x,d-2);

% Polynomial multiplier (associated with the equality constraint gxT == 0)
[r1,cr1] = polynomial(x,d-2);


l = getLebesgueMoments( d, [-xb ; xb], 1 );

Lv = jacobian(v,t) + jacobian(v,x)*f ; % linear operator A(v)

% Av <= 0 on [0 T] x X x U
con = [ sos(-Lv - gx1*s1 - gx2*s2  - gx3*s3 - gx4*s4 - gu1*s5 - gu2*s14 + t*(t-T)*st);
        sos(s1) ; sos(s2) ; sos(s3) ; sos(s4) ; sos(s5) ; sos(s14) ; sos(st) ];    
% v >= 0 on {T} x XT
con = [ con ; sos( replace(v,t,1) - gxT*r1 ) ];
% w >= v + 1 on {0} x X
con = [ con ; sos(w - replace(v,t,0) - 1 - gx1*s6 - gx2*s7  - gx3*s8 - gx4*s9) ;
        sos(s6) ; sos(s7) ; sos(s8) ; sos(s9) ];
% w >= 0 on X
con = [ con ; sos(w - gx1*s10 - gx2*s11  - gx3*s12 - gx4*s13) ;
        sos(s10) ; sos(s11) ; sos(s12) ; sos(s13)];

% Objective
obj = cw'*l;

% Solver options
options = getSolverParams(SDPsolver);
if (~isempty(options))
    mset(options);
end

% Solve
[~,~,~,res] = solvesos(con,obj,options,[cw;cv;c1;c2;c3;c4;c5;c14;ct;c6;c7;c8;c9;c10;c11;c12;c13;cr1]);
fprintf('SOS equality constraint residual = %f \n', res);

% Coefficients of w(x) and v(t,x)
cw = double(cw);
cv = double(cv);


%% Plots

tt = sdpvar(1,1);
X = sdpvar(n,1);
vv = monolist([tt;X],d);
p = cv'*vv;
p = replace(p,tt,0);
p = replace(p,X(4),0);
p = vectorize(sdisplay(p));
p = strrep(p, 'X(1)', 'X');
p = strrep(p, 'X(2)', 'Y');
p = strrep(p, 'X(3)', 'Z');

[X,Y,Z] = meshgrid(linspace(-xb(1),xb(1),100),linspace(-xb(2),xb(2),100),linspace(-xb(3),xb(3),100));
pval = eval(p);
X = X/scale_x(1); Y = Y/scale_x(2); Z = Z/scale_x(3);


h = patch(isosurface(X,Y,Z,pval,0.0));
isonormals(X,Y,Z,pval,h)
set(h,'FaceColor',[1 0 0],'EdgeColor','none'); % red
lighting phong; view(3); axis tight vis3d; camlight ;
set(gca,'CameraPosition', [21.8086   14.4348    5.6335]);
set(gca,'cameraViewAngle',10.3396);
set(gca,'CameraTarget',[0 0 0]);
title('Outer approximation of the Acrobot ROA')
xlabel('x_1'); ylabel('x_2'); zlabel('x_3')





