% *************************************************************************
% Problem:  Brockett integrator ROA - dual SOS problem
% Requires: Gloptipoly 3, Yalmip + SDP solver (tested with MOSEK and SeDuMi)
% Authors:  Milan Korda, Didier Henrion, 2013
% *************************************************************************

clear all
close all
addpath(genpath('./functions'))

% SDP solver
SDPsolver = 'mosek'; % or sedumi

% Degree of v(t,x) and w(x)
d = 6;

% Variables
n = 3; % state dimension
m = 2; % dimesion of control
x = sdpvar(n,1); % state
u = sdpvar(m,1); % control
t = sdpvar(1,1); % time

% Polynomials w(x) and v(t,x)
[w, cw] = polynomial(x,d);
[v, cv] = polynomial([t;x],d);

% Final time
T = 1;

% Dynamics
f = [ u(1) ; u(2) ; u(1)*x(2) - u(2)*x(1) ] * T; % Brocket integrator

% Target set XT = {x : gxT = 0}
gxT = x'*x;

% State constraint set X = [-xb1, xb1] x [-xb2, xb2] x [-xb3, xb3] = {x : gx_i >= 0}
xb1 = 1; xb2 = 1; xb3 = 1;
gx1 = xb1^2 - x(1)^2;
gx2 = xb2^2 - x(2)^2;
gx3 = xb3^2 - x(3)^2;

% Input constraint set U = {u : ||x|| <= 1} = {u : gu >= 0}
gu = 1 - u'*u; 


% SOS multipliers
[s1,c1] = polynomial([t;x;u],d-2);
[s2,c2] = polynomial([t;x;u],d-2);
[s3,c3] = polynomial([t;x;u],d-2);
[s4,c4] = polynomial(x,d-2);
[st,ct] = polynomial([t;x;u],d-2);
[s5,c5] = polynomial(x,d-2);
[s6,c6] = polynomial(x,d-2);
[s7,c7] = polynomial(x,d-2);
[s8,c8] = polynomial(x,d-2);
[s9,c9] = polynomial(x,d-2);
[s10,c10] = polynomial(x,d-2);

% Polynomial multiplier (associated with the equality constraint gxT == 0)
[r1,cr1] = polynomial(x,d-2);


Lv = jacobian(v,t) + jacobian(v,x)*f ; % linear operator L(v)

% Lv <= 0 on [0 T] x X x U
con = [ sos( -Lv - gx1*s1 - gx2*s2 - gx3*s8 - gu*s3 + t*(t-T)*st ) ; sos(s1) ;
        sos(s2) ; sos(s3) ; sos(st) ; sos(s8) ];

% v >= 0 on {T} x XT
con = [ con ; sos( replace(v,t,T) - gxT*r1 ) ];

% w >= v + 1 on {0} x X
con = [ con ; sos( -replace(v,t,0) + w - 1 - gx1*s4 - gx2*s5 - gx3*s9 ) ;...
    sos(s4) ; sos(s5) ; sos(s9) ];

% w >= 0 on X
con = [ con ; sos( w - gx1*s6 - gx2*s7 - gx3*s10 ) ; sos(s6) ; sos(s7) ; sos(s10) ];


% Lebesgue moments on {0} x X
%load('leb_mom.mat','leb_box_di')
l = getLebesgueMoments(d,[-xb1 -xb2 -xb3 ; xb1 xb2 xb3],1);

% Objective
obj = cw'*l; % minimization of int w d_lambda

% Solver options
options = getSolverParams(SDPsolver);
if (~isempty(options))
    mset(options);
end

% Solve
[~,~,~,res] = solvesos(con,obj,options,[cw;cv;c1;c2;c3;c4;ct;c5;c6;c7;cr1;c8;c9;c10]);
fprintf('SOS equality constraint residual = %f \n', res);

% Coefficients of w(x) and v(t,x)
cw = double(cw);
cv = double(cv);


%% Plots

X = sdpvar(n,1);
vv = monolist([t;X],d);
p = vectorize(sdisplay(replace(cv'*vv,t,0)+1));
p = strrep(p, 'X(1)', 'X');
p = strrep(p, 'X(2)', 'Y');
p = strrep(p, 'X(3)', 'Z');

[X,Y,Z] = meshgrid(linspace(-xb1,xb1,100),linspace(-xb2,xb2,100),linspace(-xb3,xb3,100));
pval = eval(p);
pval(abs(X) == xb1 | abs(Y) == xb2 | abs(Z) == xb3) = -1;

% Outer approximation
h = patch(isosurface(X,Y,Z,pval,1.0));
isonormals(X,Y,Z,pval,h)
set(h,'FaceColor',[1 0 0],'EdgeColor','none');
lighting phong ; view(3); axis vis3d ; camlight
alpha(0.4);

% True ROA
load('./functions/BrockettValueFun.mat','X','Y','Z','V');
h = patch(isosurface(X,Y,Z,V,1.0));
isonormals(X,Y,Z,V,h)
set(h,'FaceColor',[1 0 0],'EdgeColor','none');
view(3); axis vis3d; camlight; lighting phong
title('Brockett integrator ROA and its outer approixmation')
xlabel('x_1'); ylabel('x_2'); zlabel('x_3')

