% *************************************************************************
% Problem:  Brockett integrator ROA - primal truncated moment problem
% Requires: Gloptipoly 3, Yalmip + SDP solver (tested with MOSEK and SeDuMi)
% Authors:  Milan Korda, Didier Henrion, 2013
% *************************************************************************

close all
clear all
addpath(genpath('./functions'))
mset clear
mset('yalmip',true); % if true Gloptipoly calls the SDP solver through Yalmip
warning('off','YALMIP:strict')

% SDP solver
SDPsolver = 'mosek'; % or sedumi

d = 6; % degree of the polynomials w and v (relaxation order = d/2)
n = 3; % state-space dimension
m = 2; % control input dimension


% Variables
mpol('t',1);
mpol('x',n);
mpol('u',2);

mpol('tT',1);
mpol('xT',n);

mpol('t0',1)
mpol('x0',n);
mpol('xc',n);

% Assign varibles to measures
mu0 = meas([t0;x0]); % initial
muc = meas(xc);      % complementary
mut = meas([tT;xT]); % terminal
mu = meas([t;x;u]);  % trajectory

% Final time
T = 1;

% Dynamics
f = [ u(1) ; u(2) ; u(1)*x(2) - u(2)*x(1) ];


% State constraint set X = [-xb1, xb1] x [-xb2, xb2] x [-xb3, xb3]
xb1 = 1;
xb2 = 1;
xb3 = 1;

% Test functions
v = mmon([t;x],d);
v00 = mmon([t0;x0],d);
vTT = mmon([tT;xT],d);
v0 = mmon(x0,d);
vc = mmon(xc,d);

l = getLebesgueMoments(d,[-xb1 -xb2 -xb3 ; xb1 xb2 xb3]);


% Moment constraints
mom_const = [  mom( diff(v,t) + diff(v,x)*f ) - mom(vTT) + mom(v00)  == 0 ;
              mom(v0) + mom(vc) - l == 0 ];

% Support constraints
supp_const = [ t*(1-t)>=0 ;
               x(1)^2 <= xb1^2  ; x(2)^2  <= xb2^2 ;  x(3)^2  <= xb3^2 ;
               x0(1)^2 <= xb1^2 ; x0(2)^2 <= xb2^2 ;  x0(3)^2 <= xb3^2 ;
               xc(1)^2 <= xb1^2 ; xc(2)^2 <= xb2^2 ;  xc(3)^2 <= xb3^2 ;
               u'*u <= 1 ;
               xT'*xT == 0 ;
               t0 == 0 ; tT == 1 
             ];
% Objective
obj = max(mass(mu0));

% Set up the problem
P = msdp(obj, mom_const, supp_const);


% Solver options
options = getSolverParams(SDPsolver);
if (~isempty(options))
    mset(options);
end

% Solve
[status,obj,~,xx] = msol(P);
if(status < 0)
    disp('SDP could not be solved')
    return
end


% Extract coefficients of w(x) and v(t,x)
cw = xx(size(v,1)+(1:size(v0,1)));
cv = -xx(1:size(v,1));





%% Plots

tt = sdpvar(1,1); X = sdpvar(n,1);
vv = monolistYalToGlop([tt;X],d); 
p = vectorize(sdisplay(replace(cv'*vv,tt,0)+1));
p = strrep(p, 'X(1)', 'X');
p = strrep(p, 'X(2)', 'Y');
p = strrep(p, 'X(3)', 'Z');

[X,Y,Z] = meshgrid(linspace(-xb1,xb1,100),linspace(-xb2,xb2,100),linspace(-xb3,xb3,100));
pval = eval(p);
pval(abs(X) == xb1 | abs(Y) == xb2 | abs(Z) == xb3) = -1;

% Outer approximation
h = patch(isosurface(X,Y,Z,pval,1.0));
isonormals(X,Y,Z,pval,h)
set(h,'FaceColor',[1 0 0],'EdgeColor','none'); 
lighting phong ; view(3); axis vis3d ; camlight
alpha(0.4);

% True ROA
load('./functions/BrockettValueFun.mat','X','Y','Z','V');
h = patch(isosurface(X,Y,Z,V,1.0));
isonormals(X,Y,Z,V,h)
set(h,'FaceColor',[1 0 0],'EdgeColor','none');
view(3); axis vis3d ;camlight ;lighting phong
title('Brockett integrator ROA and its outer approixmation')
xlabel('x_1'); ylabel('x_2'); zlabel('x_3')







