% *************************************************************************
% Problem:  Double Integrator ROA - dual SOS problem
% Requires: Gloptipoly 3, Yalmip + SDP solver (tested with MOSEK and SeDuMi)
% Authors:  Milan Korda, Didier Henrion, 2013
% *************************************************************************

clear all
close all
addpath(genpath('./functions'))

% SDP solver
SDPsolver = 'mosek'; % or sedumi

% Degree of v(t,x) and w(x)
d = 8;

% Variables
x = sdpvar(2,1);
u = sdpvar(1,1);
t = sdpvar(1,1);

% Polynomials w(x) and v(t,x)
[w, cw] = polynomial(x,d);
[v, cv] = polynomial([t;x],d);

% Final time
T = 1; 

% Dynamics
f = [ x(2) ; u ] * T;

% Target set XT = {x | gxT = 0}
gxT = x'*x;

% State constraint set X = [-xb1, xb1] x [-xb2, xb2] = {x : gx1 >=, gx2 >= 0}
xb1 = 0.7; xb2 = 1.2;
gx1 = xb1^2 - x(1)^2;
gx2 = xb2^2 - x(2)^2;


% Input constraint set U = [-1,1] = {u : gu >= 0}
gu = 1^2 - u^2;

% SOS multipliers
[s1,c1] = polynomial([t;x;u],d-2);
[s2,c2] = polynomial([t;x;u],d-2);
[s3,c3] = polynomial([t;x;u],d-2);
[s4,c4] = polynomial(x,d-2);
[st,ct] = polynomial([t;x;u],d-2);
[s5,c5] = polynomial(x,d-2);
[s6,c6] = polynomial(x,d-2);
[s7,c7] = polynomial(x,d-2);

% Polynomial multiplier (associated with the equality constraint gxT == 0)
[r1,cr1] = polynomial(x,d-2);


Lv = jacobian(v,t) + jacobian(v,x)*f ; % linear operator Lv

% AV <= 0 on [0 T] x X x U
con = [ sos( -Lv - gx1*s1 - gx2*s2 - gu*s3 + t*(t-1)*st ) ; sos(s1) ;
        sos(s2) ; sos(s3) ; sos(st) ];    
% v >= 0 on {T} x XT
con = [ con ; sos( replace(v,t,1) - gxT*r1 ) ];
% w >= V + 1 on {0} x X
con = [ con ; sos( -replace(v,t,0) + w - 1 - gx1*s4 - gx2*s5 ) ;...
        sos(s4) ; sos(s5) ];   
% w >= 0 on X
con = [ con ; sos( w - gx1*s6 - gx2*s7 ) ; sos(s6) ; sos(s7) ];

% Lebesgue moments on X
l = getLebesgueMoments(d,[-xb1 -xb2;xb1 xb2],1);

% Objective
obj = cw'*l; % minimization of int W d_lambda


% Solver options
options = getSolverParams(SDPsolver);
if (~isempty(options))
    mset(options);
end


[~,~,~,res] = solvesos(con,obj,options,[cw;cv;c1;c2;c3;c4;ct;c5;c6;c7;cr1]);
fprintf('Residuals %f \n',res);

% Coefficients of w(x) and v(t,x)
cw = double(cw);
cv = double(cv);


%% Plot

% Surface plot of w(x)
figure
X = sdpvar(1,1); Y = sdpvar(1,1);
vv = monolist([X;Y],d);
p = vectorize(sdisplay(cw'*vv)); %w(x)
[X,Y] = meshgrid(-xb1:0.05:xb1,-xb2:0.05:xb2);
Z = eval(p);
surf(X,Y,Z)
xlabel('x_1') ; ylabel('x_2'); legend('w(x)')


% Level-set plot of { x : v(0,x) >= 0 }
tt = sdpvar(1,1); X = sdpvar(1,1); Y = sdpvar(1,1);
vv = monolist([tt;X;Y],d);
p = vectorize(sdisplay(replace(cv'*vv+1,tt,0))); % v(0,x) + 1
[X,Y] = meshgrid(-xb1:0.005:xb1,-xb2:0.005:xb2);
Z = eval(p);
contour(X,Y,Z, [1 1], '-b', 'linewidth',2); hold on

% True ROA
[X,Y] = meshgrid(-xb1:0.005:xb1,-xb2:0.005:xb2);
Z = zeros(size(X));
for i = 1:size(X,1)
    for j = 1:size(X,2)
        Z(i,j) = doubleIntegValueFun([X(i,j),Y(i,j)]);
    end
end
[xv h] = contour(X,Y,Z,[T T], '-r', 'linewidth',2);
xlabel('x_1') ; ylabel('x_2');
legend('Outer', 'True');








