% *************************************************************************
% Problem:  Double integrator ROA - primal truncated moment problem
% Requires: Gloptipoly 3, Yalmip + SDP solver (tested with MOSEK and SeDuMi)
% Authors:  Milan Korda, Didier Henrion, 2013
% *************************************************************************

mset clear
close all
clear all
mset('yalmip',true); % if true Gloptipoly calls the SDP solver through Yalmip
warning('off','YALMIP:strict')

% SDP solver
SDPsolver = 'mosek'; % or sedumi

d = 8; % degree of the polynomials w and v (relaxation order = d/2)
n = 2; % state-space dimension

% Variables
mpol('t',1);
mpol('x',n);
mpol('u',1);

mpol('tT',1);
mpol('xT',n);

mpol('t0',1)
mpol('x0',n);
mpol('x_hat',n);

% Assign varibles to measures
mu0 = meas([t0;x0]);  % initial
mu_hat = meas(x_hat); % complementary
muT = meas([tT;xT]);  % terminal
mu = meas([t;x;u]);   % occupation

% Final time
T = 1; 

% Dynamics
f = [ x(2) ; u ]*T ; % double integrator

% Constraint set X = [-xb1,xb1] x [-xb2,xb2]
xb1 = 0.7;
xb2 = 1.2;

% Test functions
v = mmon([t;x],d);
v0 = mmon(x0,d);
v_hat = mmon(x_hat,d);
v00 = mmon([t0;x0],d);
vT = mmon([tT;xT],d);


% Moments of the lebesgue measure on X
l = getLebesgueMoments(d,[-xb1 -xb2 ; xb1 xb2]);


% Moment constraints
mom_const = [  mom( diff(v,t) + diff(v,x)*f ) - mom(vT) + mom(v00)  == 0 ;... % Liouville
               mom(v0) + mom(v_hat) - l == 0    % mu + m_hat = Lebesgue
            ];

% Support constraints
supp_const = [ t*(1-t)>=0 ; x(1)^2 <= xb1^2 ; x(2)^2 <= xb2^2 ; ...
               x0(1)^2 <= xb1^2 ; x0(2)^2 <= xb2^2 ; ...
               x_hat(1)^2 <= xb1^2 ; x_hat(2)^2 <= xb2^2 ; ...
               u^2 <= 1 ;
               xT == 0 ;
               t0 == 0 ; tT == 1
             ];
% Objective         
obj = max(mass(mu0));

% Build the problem
P = msdp(obj, mom_const, supp_const);

% Solver options
options = getSolverParams(SDPsolver);
if (~isempty(options))
    mset(options);
end

% Solve
[status,obj,~,xx] = msol(P);
if(status < 0)
    disp('SDP could not be solved')
    return
end


% Extract coefficients of w(x) and v(t,x)
cw = xx(size(v,1)+(1:size(v0,1)));
cv = -xx(1:size(v,1));


%% Plot

% Surface plot of w(x)
figure
X = sdpvar(1,1); Y = sdpvar(1,1);
vv = monolist([X;Y],d);
p = vectorize(sdisplay(cw'*vv)); %w(x)
[X,Y] = meshgrid(-xb1:0.05:xb1,-xb2:0.05:xb2);
Z = eval(p);
surf(X,Y,Z)
xlabel('x_1') ; ylabel('x_2'); legend('w(x)')


% Level-set plot of { x : v(0,x) >= 0 }
tt = sdpvar(1,1); X = sdpvar(1,1); Y = sdpvar(1,1);
vv = monolistYalToGlop([tt;X;Y],d);
p = vectorize(sdisplay(replace(cv'*vv+1,tt,0))); % v(0,x) + 1
[X,Y] = meshgrid(-xb1:0.005:xb1,-xb2:0.005:xb2);
Z = eval(p);
contour(X,Y,Z, [1 1], '-b', 'linewidth',2); hold on

% True ROA
[X,Y] = meshgrid(-xb1:0.005:xb1,-xb2:0.005:xb2);
Z = zeros(size(X));
for i = 1:size(X,1)
    for j = 1:size(X,2)
        Z(i,j) = doubleIntegValueFun([X(i,j),Y(i,j)]);
    end
end
[xv h] = contour(X,Y,Z,[T T], '-r', 'linewidth',2);
xlabel('x_1') ; ylabel('x_2');
legend('Outer', 'True')

