% *************************************************************************
% Problem:  Univariate system ROA - dual SOS problem
% Requires: Gloptipoly 3, Yalmip + SDP solver (tested with MOSEK and SeDuMi)
% Authors:  Milan Korda, Didier Henrion, 2013
% *************************************************************************


clear all
close all
addpath(genpath('./functions'))

% SDP solver
SDPsolver = 'mosek'; % or sedumi

% Degree of v(t,x) and w(x)
d = 10;

t = sdpvar(1);
x = sdpvar(1);
[w, cw] = polynomial(x,d);
[v,cv] = polynomial([t x],d);


% Constraint set X = [-xb xb]
xb = 1;

% Dynamics with time scaled to 1
T = 100;
f = x*(x^2-0.5^2) * T;

% X = {x | gx >= 0}
gx = -(x-xb)*(x+xb);

% XT = {x | gxT >= 0}
gxT = -(x-0.1)*(x+0.1);


% SOS multipliers
[s1,c1] = polynomial([t x],d-2);
[s2,c2] = polynomial(x,d-2);
[s3,c3] = polynomial(x,d-2);
[st,ct] = polynomial([t x],d-2);
[s4,c4] = polynomial(x,d-2);


Lv = jacobian(v,t) + jacobian(v,x)*f;

% Constraints (Note that the dynamics was scaled by T, so there T = 1)
con = [ sos( -Lv - gx*s1 + t*(t-1)*st ) ; sos(s1) ; sos(st) ];     % Lv <= 0 on [0,T] x X
con = [ con ; sos( replace(v,t,1) - gxT*s2 ) ; sos(s2) ];          % v >= 0 {T} x XT
con = [ con ; sos( -replace(v,t,0) + w - 1 - gx*s3 ) ; sos(s3) ];  % w  >= v + 1 on {0} x X
con = [ con ; sos( w - gx*s4 ) ; sos(s4)];                         % w >= 0 on X


% Lebesgue moments on X = [-xb xb]
l = getLebesgueMoments(d,[-xb;xb]);

% Objective
obj = cw'*l; % minimization of int w d_lambda

% Solver options
options = getSolverParams(SDPsolver);
if (~isempty(options))
    mset(options);
end

% Solve
solvesos(con,obj,options,[cw;cv;c1;c2;c3;ct;c4]);

% Coefficients of w
cw = double(cw);


%% Plots
X = -xb:0.001:xb;
Z = polyval(cw(end:-1:1),X); 
plot(X,Z, 'Color',0*[1 1 1], 'LineWidth',2); hold on


% True ROA
xr = 0.5;
stairs(X,[ zeros(size(-xb:0.001:-xr)) ones(size(-xr+0.001:0.001:xr)) zeros(size(xr+0.001:0.001:xb)) ],...
         'Color',0*[1 1 1],'LineStyle','--', 'LineWidth',1.4); hold on
     
legend('w(x)','Indicator of X_0')

ylabel(' ');
set(gca,'FontSize',14);
set(gca,'XTick',-xb:0.2:xb);
set(gca,'XTickLabel',-xb:0.2:xb);
set(gca,'YTick',-xb:0.2:1.5*xb);
set(gca,'YTickLabel',-xb:0.2:1.5*xb);
xlabel('x');
grid

