% *************************************************************************
% Problem:  Univariate system ROA - solves the primal truncated moment problem
% Requires: Gloptipoly 3, Yalmip + SDP solver (tested with MOSEK and SeDuMi)
% Authors:  Milan Korda, Didier Henrion, 2013
% *************************************************************************

clear all
close all
addpath(genpath('./functions'))
mset clear
mset('yalmip',true); % if true Gloptipoly calls the SDP solver through Yalmip
warning('off','YALMIP:strict')

% SDP solver
SDPsolver = 'mosek'; % or sedumi


% Variables
mpol('t',1);
mpol('x',1);

mpol('tT',1);
mpol('xT',1);

mpol('t0',1)
mpol('x0',1);
mpol('x_hat',1);

% Assign varibles to measures
mu0 = meas([t0;x0]);   % initial
muc = meas(x_hat);     % complementary
muT = meas([tT ; xT]); % terminal
mu = meas([t x]);      % occupation

% Final time
T = 100; 

% Dynamics with time scaled to 1
f = x*(x^2-0.5^2) * T;

% Constraint set X = [-xb,xb]
xb = 1;
gx = -(x-xb)*(x+xb);

% XT = [-epsilon,epsilon]
epsilon = 0.1;

% Degree
d = 10;

% Test functions
v = mmon([t;x],d);
v0 = mmon(x0,d);
vc = mmon(x_hat,d);
vt = mmon(xT,d);
v00 = mmon([t0;x0],d);
vtt = mmon([tT;xT],d);


% Moments of the Lebesgue measure on X
l = getLebesgueMoments(d,[-xb;xb]);


% Moment constraints
mom_const = [ mom(diff(v,[t x])*[1;f]) - mom(vtt) + mom(v00) == 0 ; 
              mom(v0) + mom(vc) - l == 0 ];
% Support constraints          
supp_const = [ t*(1-t) >= 0 ; x^2 <= xb^2 ; x0^2 <= xb^2 ; x_hat^2 <= xb^2
               xT^2 <= epsilon^2;
               t0 == 0; tT == 1;
             ];
% Objective           
obj = max(mass(mu0));

P = msdp(obj,mom_const,supp_const);

% Solver options
options = getSolverParams(SDPsolver);
if (~isempty(options))
    mset(options);
end

% Solve
[status,obj,~,xx] = msol(P);
if(status < 0)
    disp('SDP could not be solved')
    return
end

% Extract coefficients of w(x) and v(t,x)
cw = xx(size(v,1)+(1:size(v0,1)));
cv = -xx(1:size(v,1));


%% Plots
X = -xb:0.001:xb;
Z = polyval(cw(end:-1:1),X); 
plot(X,Z, 'Color',0*[1 1 1], 'LineWidth',2); hold on


% True ROA
xr = 0.5;
stairs(X,[ zeros(size(-xb:0.001:-xr)) ones(size(-xr+0.001:0.001:xr)) zeros(size(xr+0.001:0.001:xb)) ],...
         'Color',0*[1 1 1],'LineStyle','--', 'LineWidth',1.4); hold on
     
legend('w(x)','Indicator of X_0')

ylabel(' ');
set(gca,'FontSize',14);
set(gca,'XTick',-xb:0.2:xb);
set(gca,'XTickLabel',-xb:0.2:xb);
set(gca,'YTick',-xb:0.2:1.5*xb);
set(gca,'YTickLabel',-xb:0.2:1.5*xb);
xlabel('x');
grid






