% *************************************************************************
% Problem:  Van der Pol oscillator ROA - dual SOS problem
% Requires: Gloptipoly 3, Yalmip + SDP solver (tested with MOSEK and SeDuMi) 
% Authors:  Milan Korda, Didier Henrion, 2013
% *************************************************************************


clear all
close all
addpath(genpath('./functions'))

% SDP solver
SDPsolver = 'mosek'; % or sedumi

% Degree of polynomials w(x) and v(t,x)
d = 10;

% Variables
x = sdpvar(2,1);
t = sdpvar(1,1);

% Define polynomials w(x) and v(t,x)
[w cw] = polynomial(x,d);
[v,cv] = polynomial([t;x],d);

% Constraint set X = {x : ||x||_inf <= xb}
xb = 1.2;

% Final time
T = 100;

% Dynamics (scaled by final time)
f = -[ 2*x(2) ; -0.8*x(1) - 10*(x(1)^2-0.20)*x(2) ] * T;

%X_T
gxT = (0.1^2 - x'*x);

% [0,T] x X
gx1 = xb^2 - x(1)^2;
gx2 = xb^2 - x(2)^2;


% SOS multipliers
[s1,c1] = polynomial([t;x],d-2);
[s2,c2] = polynomial([t;x],d-2);
[s3,c3] = polynomial(x,d-2);
[s4,c4] = polynomial(x,d-2);
[s5,c5] = polynomial(x,d-2);
[st,ct] = polynomial([t;x],d-2);
[s6,c6] = polynomial(x,d-2);
[s7,c7] = polynomial(x,d-2);


% Lebesgue moments on X
l = getLebesgueMoments(d,[-xb -xb ; xb xb],1);

Lv = jacobian(v,t) + jacobian(v,x)*f;

% Constraints (Note that the dynamics was scaled by T, so there T = 1)
con = [ sos( -Lv - gx1*s1 - gx2*s2 + t*(t-1)*st ) ; sos(s1) ; sos(s2) ; sos(st) ];    % Lv <= 0 on [0 T] x X
con = [ con ; sos( replace(v,t,1) - gxT*s3 ) ; sos(s3) ];                             % v >= 0 on {T} x X_T
con = [ con ; sos( -replace(v,t,0) + w - 1 - gx1*s4 - gx2*s5 ) ; sos(s4) ; sos(s5) ]; % w >= v + 1 on {0} x X
con = [ con ; sos( w - gx1*s6 - gx2*s7 ) ; sos(s6) ; sos(s7) ];                       % w >= 0 on X

% Objective
obj = cw'*l; % minimization of int w d_lambda

% Solver parameteres
options = getSolverParams(SDPsolver);
if (~isempty(options))
    mset(options);
end

% Solve
solvesos(con,obj,options,[cw;cv;c1;c2;c3;c4;c5;ct;c6;c7]);

% Retrieve coefficients of w and V
cw = double(cw);
cv = double(cv);


%% Plots

% Surface plot of w(x)
X = sdpvar(1,1); Y = sdpvar(1,1);
vv = monolist([X;Y],d);
p = vectorize(sdisplay(cw'*vv)); %w(x)
[X,Y] = meshgrid(-xb:0.05:xb,-xb:0.05:xb);
Z = eval(p);
surf(X,Y,Z)
title('Polynomial w(x)')
xlabel('x_1') ; ylabel('x_2'); legend('w(x)')

% Level-set plot of { x : v(0,x) >= 0 } and the true ROA
figure
X = sdpvar(1,1); Y = sdpvar(1,1);
vv = monolist([t;X;Y],d);
p = vectorize(sdisplay(replace(cv'*vv,t,0) + 1)); % v(0,x) + 1
[X,Y] = meshgrid(-xb:0.005:xb,-xb:0.005:xb);
Z = eval(p);
contour(X,Y,Z, [1 1], '-b', 'linewidth',2); hold on


% Simulate trajectory with reversed time to get the boundary of the true ROA
f_vpo =  @(t,x)([ 2*x(2) ; -0.8*x(1) - 10*(x(1)^2-0.21)*x(2) ]);
[~, xv] = ode45(f_vpo,0:0.01:100,[0.1;0.1]);

plot(xv(2000:2574,1),xv(2000:2574,2),'-r','LineWidth',2)
xlabel('x_1'); ylabel('x_2');
legend('Outer', 'True')
title('Van der Pol oscillator ROA')




