% *************************************************************************
% Problem:  Van der Pol ROA - primal truncated moment problem
% Requires: Gloptipoly 3, Yalmip + SDP solver (tested with MOSEK and SeDuMi)
% Authors:  Milan Korda, Didier Henrion, 2013
% *************************************************************************

close all
clear all
addpath(genpath('./functions'))
mset clear
mset('yalmip',true); % if true Gloptipoly calls the SDP solver through Yalmip
warning('off','YALMIP:strict')

% SDP solver
SDPsolver = 'sedumi'; % or sedumi


% Degree
d = 10; 

% State-space dimension
n = 2; 

% Variables
mpol('t',1); 
mpol('x',n);
mpol('tT',1);
mpol('xT',n);
mpol('t0',1)
mpol('x0',n);
mpol('x_hat',n);

% Assign varibles to measures
mu0 = meas([t0;x0]); % initial
muc = meas(x_hat);   % complementary
mut = meas([tT;xT]); % terminal
mu = meas([t;x]);    % trajectory

% Final time
T = 100; 

% Target set X_T = {x : ||x||_2 <= epsilon}
epsilon = 0.1; 

% Dynamics
f = -[ 2*x(2) ; -0.8*x(1) - 10*(x(1)^2-0.21)*x(2) ] * T;

% State constraint set X = [-xb xb]^2
xb = 1.2;

v = mmon([t;x],d);
v00 = mmon([t0;x0],d);
vTT = mmon([tT;xT],d);
v0 = mmon(x0,d);
vc = mmon(x_hat,d);

% Reference measure on X = [-xb, xb]^2
l = getLebesgueMoments(d,[-xb -xb ; xb xb]);

% Moment constraints
mom_const = [  mom( diff(v,t) + diff(v,x)*f ) - mom(vTT) + mom(v00)  == 0 ;
               mom(v0) + mom(vc) - l == 0 ];

% Support constraints
supp_const = [ t*(1-t)>=0 ; x(1)^2 <= xb^2 ; x(2)^2 <= xb^2 ;
               x0(1)^2 <= xb^2 ; x0(2)^2 <= xb^2 ; x_hat(1)^2 <= xb^2 ; x_hat(2)^2 <= xb^2 ;
               xT'*xT <= epsilon^2 ;
               t0 == 0 ; tT == 1 
             ];

% Objective
obj = max(mass(mu0));


% Build the problem
P = msdp(obj, mom_const, supp_const);

% Solver options
options = getSolverParams(SDPsolver);
if (~isempty(options))
    mset(options);
end

% Solve
[status,obj,~,xx] = msol(P);
if(status < 0)
    disp('SDP could not be solved')
    return
end


% Extract coefficients of w(x) and v(t,x)
cw = xx(size(v,1)+(1:size(v0,1)));
cv = -xx(1:size(v,1));




%% Plots

% Surface plot of w(x)
X = sdpvar(1,1); Y = sdpvar(1,1);
vv = monolist([X;Y],d);
p = vectorize(sdisplay(cw'*vv)); %w(x)
[X,Y] = meshgrid(-xb:0.05:xb,-xb:0.05:xb);
Z = eval(p);
surf(X,Y,Z)
title('Polynomial w(x)')
xlabel('x_1') ; ylabel('x_2'); legend('w(x)')

% Level-set plot of { x : v(0,x) >= 0 } and the true ROA
figure
tt = sdpvar(1,1); X = sdpvar(1,1); Y = sdpvar(1,1);
vv = monolistYalToGlop([tt;X;Y],d);
p = vectorize(sdisplay(replace(cv'*vv,tt,0) + 1)); % v(0,x) + 1
[X,Y] = meshgrid(-xb:0.005:xb,-xb:0.005:xb);
Z = eval(p);
contour(X,Y,Z, [1 1], '-b', 'linewidth',2); hold on


% Simulate trajectory with reversed time to get the boundary of the true ROA
f_vpo =  @(t,x)([ 2*x(2) ; -0.8*x(1) - 10*(x(1)^2-0.21)*x(2) ]);
[~, xv] = ode45(f_vpo,0:0.01:100,[0.1;0.1]);

plot(xv(2000:2574,1),xv(2000:2574,2),'-r','LineWidth',2)
xlabel('x_1'); ylabel('x_2');
legend('Outer', 'True')
title('Van der Pol oscillator ROA')





